import cv2
import numpy as np
import socket

#INPUT SEND DATA
IP = '10.68.173.10' #paste your own IP here
port = 6002
#----------------------------------------


# capture video
cap = cv2.VideoCapture(0, cv2.CAP_DSHOW)

#segment one color (or more)
#setting color limits (for red)

colorLowL = np.array([0, 100, 20], np.uint8)
colorUpL = np.array([10, 255, 255], np.uint8)

# define the kernel
kernel = np.ones((5, 5), "uint8")

# define the color name
cName = 'red'
cOut = (0, 0, 255)
run = True
minArea = 1000

try:
    while (cap.isOpened()):
        ret, RGBframe = cap.read()

        #FIRST, COLOUR SEGMENTATION
        width = RGBframe.shape[1]
        height = RGBframe.shape[0]

        frame = (width, height)

        HSVframe = cv2.cvtColor(RGBframe, cv2.COLOR_BGR2HSV)

        # define the color mask
        colorMask = cv2.inRange(HSVframe, colorLowL, colorUpL)

        #
        colorMask = cv2.dilate(colorMask, kernel)
        colorRes = cv2.bitwise_and(RGBframe, RGBframe, mask=colorMask)

        #SECOND, GENERATE CONTOURS FOR THE SEGMENTED COLOUR ONLY
        gray = cv2.cvtColor(colorRes, cv2.COLOR_BGR2GRAY)
        blurred = cv2.GaussianBlur(gray, (5, 5), 0)
        thresh = cv2.threshold(gray, 4, 255, cv2.THRESH_BINARY)[1]

        contours, hierarchy = cv2.findContours(thresh, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)

        print(contours)

        areas = []
        centers = []
        items = []

        for c in contours:

            area = cv2.contourArea(c)

            if (area > minArea):
                detection = True
                M = cv2.moments(c)
                cX = int(M["m10"] / M["m00"])
                cY = int(M["m01"] / M["m00"])

                center = [(cX, cY)]
                centers.append(center)
                items.append(c)

                approx = cv2.approxPolyDP(c, 0.0001 * cv2.arcLength(c, True), True)
                cv2.drawContours(RGBframe, [approx], 0, (0, 0, 0), 1)
                areas.append(area)


        cv2.imshow("contour approximation", RGBframe)
        if cv2.waitKey(10) & 0xFF == ord('q'):
            break

        items = len(items)
        # LIVE DATA STREAMING MODULE

        n = 'contour approximation'  # input script color here
        d = detection
        c = centers
        e = contours
        a = areas
        s = 'no shape'
        i = items
        f = frame

        template = str(
            'name:{}|detection:{}|center:{}|edge:{}|area:{}|shape:{}|items:{}|frame:{}'.format(n, d, c, e, a, s, i, f))


        def UDP_client(IP, port, message):
            sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            sock.sendto(bytes(message, "utf-8"), (IP, port))


        UDP_client(IP, port, template)


    cap.release()
    cv2.destroyAllWindows()


except Exception as e:
    print(e)

